searchData={"items":[{"type":"module","title":"Cldr.Formatter.Plugin","doc":"A CLDR/ICU message formatter for mix format plugin.","ref":"Cldr.Formatter.Plugin.html"},{"type":"module","title":"Usage - Cldr.Formatter.Plugin","doc":"Add `Cldr.Formatter.Plugin` to the `:plugins` in `.formatter.exs`:\n```\n[\n  plugins: [Cldr.Formatter.Plugin]\n  # ...\n]\n```\nAnd then when you run `mix format`, it'll format CLDR messages `~M`\nsigils.","ref":"Cldr.Formatter.Plugin.html#module-usage"},{"type":"module","title":"Cldr.Gettext.Interpolation","doc":"As of [Gettext 0.19](https://hex.pm/packages/gettext/0.19.0), `Gettext`\nsupports user-defined [interpolation modules](https://hexdocs.pm/gettext/Gettext.html#module-backend-configuration).\nThis makes it easy to combine the power of ICU message formats with the\nbroad `gettext` ecosystem and the inbuilt support for `gettext`\nin [Phoenix](https://hex.pm/packages/phoenix).\n\nThe documentation for [Gettext](https://hexdocs.pm/gettext/Gettext.html#content)\nshould be followed with considerations in mind:\n\n1. A Gettext backend module should use the `:interpolation` option\n   defined referring to the `ex_cldr_messages` backend you have defined.\n2. The message format is in the ICU message format (instead of the Gettext format).\n\n#","ref":"Cldr.Gettext.Interpolation.html"},{"type":"module","title":"Defining a Gettext Interpolation Module - Cldr.Gettext.Interpolation","doc":"Any [ex_cldr](https://hex.pm/packages/ex_cldr) [backend module](https://hexdocs.pm/ex_cldr/readme.html#backend-module-configuration) that has a `Cldr.Message` provider configured can be used as an interpolation module. Here is an example:\n```elixir\n# CLDR backend module\ndefmodule MyApp.Cldr do\n  use Cldr,\n    locales: [\"en\", \"fr\", \"ja\", \"he\", \"th\", \"ar\"],\n    default_locale: \"en\",\n    providers: [Cldr.Number, Cldr.DateTime, Cldr.Unit, Cldr.List, Cldr.Calendar, Cldr.Message],\n    gettext: MyApp.Gettext,\n    message_formats: %{\n      USD: [format: :long]\n    }\nend\n\n# Define an interpolation module for ICU messages\ndefmodule MyApp.Gettext.Interpolation do\n  use Cldr.Gettext.Interpolation, cldr_backend: MyApp.Cldr\n\nend\n\n# Define a gettext module with ICU message interpolation\ndefmodule MyApp.Gettext do\n  use Gettext, otp_app: :ex_cldr_messages, interpolation: MyApp.Gettext.Interpolation\nend\n\n```\nNow you can proceed to use `Gettext` in the normal manner, most\ntypically with the `gettext/3` macro.","ref":"Cldr.Gettext.Interpolation.html#module-defining-a-gettext-interpolation-module"},{"type":"module","title":"Cldr.Message","doc":"Implements the [ICU Message Format](http://userguide.icu-project.org/formatparse/messages)\nwith functions to parse and interpolate messages.","ref":"Cldr.Message.html"},{"type":"function","title":"Cldr.Message.bindings/1","doc":"Extract the binding names from an ICU message.","ref":"Cldr.Message.html#bindings/1"},{"type":"function","title":"Arguments - Cldr.Message.bindings/1","doc":"* `message` is a CLDR message in binary or\n  parsed form.\n\n#","ref":"Cldr.Message.html#bindings/1-arguments"},{"type":"function","title":"Returns - Cldr.Message.bindings/1","doc":"* A list of binding names as strings or\n\n* `{:error, {exception, reason}}`\n\n#","ref":"Cldr.Message.html#bindings/1-returns"},{"type":"function","title":"Examples - Cldr.Message.bindings/1","doc":"iex> Cldr.Message.bindings \"This {variable} is in the message\"\n     [\"variable\"]","ref":"Cldr.Message.html#bindings/1-examples"},{"type":"function","title":"Cldr.Message.canonical_message/2","doc":"Formats a message into a canonical form.\n\nThis allows for messages to be compared\ndirectly, or using `Cldr.Message.jaro_distance/3`.","ref":"Cldr.Message.html#canonical_message/2"},{"type":"function","title":"Arguments - Cldr.Message.canonical_message/2","doc":"* `message` is a CLDR message in binary form.\n\n* `options` is a keyword list of options. The\n  default is `[]`.","ref":"Cldr.Message.html#canonical_message/2-arguments"},{"type":"function","title":"Options - Cldr.Message.canonical_message/2","doc":"* `:trim` determines if the message is trimmed\n  of whitespace before formatting. The default is\n  `true`.\n\n* `:pretty` determines if the message if\n  formatted with indentation to aid readability.\n  The default is `false`.","ref":"Cldr.Message.html#canonical_message/2-options"},{"type":"function","title":"Returns - Cldr.Message.canonical_message/2","doc":"* `{ok, canonical_message}` where `canonical_message`\n  is a string or\n\n* `{:error, {exception, reason}}`","ref":"Cldr.Message.html#canonical_message/2-returns"},{"type":"function","title":"Examples - Cldr.Message.canonical_message/2","doc":"iex> Cldr.Message.canonical_message \"{greeting } to you!\"\n    {:ok, \"{greeting} to you!\"}","ref":"Cldr.Message.html#canonical_message/2-examples"},{"type":"function","title":"Cldr.Message.canonical_message!/2","doc":"Formats a message into a canonical form\nor raises if the message cannot be parsed.\n\nThis allows for messages to be compared\ndirectly, or using `Cldr.Message.jaro_distance/3`.","ref":"Cldr.Message.html#canonical_message!/2"},{"type":"function","title":"Arguments - Cldr.Message.canonical_message!/2","doc":"* `message` is a CLDR message in binary form.\n\n* `options` is a keyword list of options. The\n  default is `[]`.","ref":"Cldr.Message.html#canonical_message!/2-arguments"},{"type":"function","title":"Options - Cldr.Message.canonical_message!/2","doc":"* `:trim` determines if the message is trimmed\n  of whitespace before formatting. The default is\n  `true`.\n\n* `:pretty` determines if the message if\n  formatted with indentation to aid readability.\n  The default is `false`.","ref":"Cldr.Message.html#canonical_message!/2-options"},{"type":"function","title":"Returns - Cldr.Message.canonical_message!/2","doc":"* `canonical_message` as a string or\n\n* raises an exception","ref":"Cldr.Message.html#canonical_message!/2-returns"},{"type":"function","title":"Examples - Cldr.Message.canonical_message!/2","doc":"iex> Cldr.Message.canonical_message! \"{greeting } to you!\"\n    \"{greeting} to you!\"","ref":"Cldr.Message.html#canonical_message!/2-examples"},{"type":"function","title":"Cldr.Message.format/3","doc":"Format a message in the [ICU Message Format](https://unicode-org.github.io/icu/userguide/format_parse/messages)\ninto a string.\n\nThe ICU Message Format uses message `\"pattern\"` strings with\nvariable-element placeholders enclosed in {curly braces}. The\nargument syntax can include formatting details, otherwise a\ndefault format is used.","ref":"Cldr.Message.html#format/3"},{"type":"function","title":"Arguments - Cldr.Message.format/3","doc":"* `bindings` is a list or map of arguments that\n  are used to replace placeholders in the message.\n\n* `options` is a keyword list of options.","ref":"Cldr.Message.html#format/3-arguments"},{"type":"function","title":"Options - Cldr.Message.format/3","doc":"* `backend` is any `Cldr` backend. That is, any module that\n  contains `use Cldr`.\n\n* `:locale` is any valid locale name returned by `Cldr.known_locale_names/0`\n  or a `t:Cldr.LanguageTag` struct.  The default is `Cldr.get_locale/0`.\n\n* `:trim` determines if the message is trimmed\n  of whitespace before formatting. The default is\n  `false`.\n\n* `:allow_positional_args` determines if position arguments\n  are permitted. Positional arguments are in the format\n  `{0}` in the message. The default is `true`.\n\n* All other aptions are passed to the `to_string/2`\n  function of a formatting module.","ref":"Cldr.Message.html#format/3-options"},{"type":"function","title":"Returns - Cldr.Message.format/3","doc":"* `{:ok, formatted_mesasge}` or\n\n* `{:error, {module, reason}}`","ref":"Cldr.Message.html#format/3-returns"},{"type":"function","title":"Examples - Cldr.Message.format/3","doc":"iex> Cldr.Message.format \"{greeting} to you!\", greeting: \"Good morning\"\n    {:ok, \"Good morning to you!\"}","ref":"Cldr.Message.html#format/3-examples"},{"type":"function","title":"Cldr.Message.format!/3","doc":"","ref":"Cldr.Message.html#format!/3"},{"type":"function","title":"Cldr.Message.format_list/3","doc":"","ref":"Cldr.Message.html#format_list/3"},{"type":"function","title":"Cldr.Message.format_list!/3","doc":"","ref":"Cldr.Message.html#format_list!/3"},{"type":"function","title":"Cldr.Message.format_to_iolist/3","doc":"Format a message in the [ICU Message Format](https://unicode-org.github.io/icu/userguide/format_parse/messages)\ninto an iolist.\n\nThe ICU Message Format uses message `\"pattern\"` strings with\nvariable-element placeholders enclosed in {curly braces}. The\nargument syntax can include formatting details, otherwise a\ndefault format is used.","ref":"Cldr.Message.html#format_to_iolist/3"},{"type":"function","title":"Arguments - Cldr.Message.format_to_iolist/3","doc":"* `bindings` is a list or map of arguments that\n  are used to replace placeholders in the message.\n\n* `options` is a keyword list of options.","ref":"Cldr.Message.html#format_to_iolist/3-arguments"},{"type":"function","title":"Options - Cldr.Message.format_to_iolist/3","doc":"* `backend` is any `Cldr` backend. That is, any module that\n  contains `use Cldr`.\n\n* `:locale` is any valid locale name returned by `Cldr.known_locale_names/0`\n  or a `t:Cldr.LanguageTag` struct.  The default is `Cldr.get_locale/0`.\n\n* `:trim` determines if the message is trimmed\n  of whitespace before formatting. The default is\n  `false`.\n\n* `:allow_positional_args` determines if position arguments\n  are permitted. Positional arguments are in the format\n  `{0}` in the message. The default is `true`.\n\n* All other aptions are passed to the `to_string/2`\n  function of a formatting module.","ref":"Cldr.Message.html#format_to_iolist/3-options"},{"type":"function","title":"Returns - Cldr.Message.format_to_iolist/3","doc":"* `{:ok, formatted_mesasge}` or\n\n* `{:error, {module, reason}}`","ref":"Cldr.Message.html#format_to_iolist/3-returns"},{"type":"function","title":"Examples - Cldr.Message.format_to_iolist/3","doc":"iex> Cldr.Message.format_to_iolist \"{greeting} to you!\", greeting: \"Good morning\"\n    {:ok, [\"Good morning\", \" to you!\"], [\"greeting\"], []}","ref":"Cldr.Message.html#format_to_iolist/3-examples"},{"type":"function","title":"Cldr.Message.jaro_distance/3","doc":"Returns the [Jaro distance](https://en.wikipedia.org/wiki/Jaro–Winkler_distance)\nbetween two messages.\n\nThis allows for fuzzy matching of message\nwhich can be helpful when a message string\nis changed but the semantics remain the same.","ref":"Cldr.Message.html#jaro_distance/3"},{"type":"function","title":"Arguments - Cldr.Message.jaro_distance/3","doc":"* `message1` is a CLDR message in binary form.\n\n* `message2` is a CLDR message in binary form.\n\n* `options` is a keyword list of options. The\n  default is `[]`.","ref":"Cldr.Message.html#jaro_distance/3-arguments"},{"type":"function","title":"Options - Cldr.Message.jaro_distance/3","doc":"* `:trim` determines if the message is trimmed\n  of whitespace before formatting. The default is\n  `false`.","ref":"Cldr.Message.html#jaro_distance/3-options"},{"type":"function","title":"Returns - Cldr.Message.jaro_distance/3","doc":"* `{ok, distance}` where `distance` is a float value between 0.0\n  (equates to no similarity) and 1.0 (is an\n  exact match) representing Jaro distance between `message1`\n  and `message2` or\n\n* `{:error, {exception, reason}}`","ref":"Cldr.Message.html#jaro_distance/3-returns"},{"type":"function","title":"Examples - Cldr.Message.jaro_distance/3","doc":"iex> Cldr.Message.jaro_distance \"{greetings} to you!\", \"{greeting} to you!\"\n    {:ok, 0.9824561403508771}","ref":"Cldr.Message.html#jaro_distance/3-examples"},{"type":"function","title":"Cldr.Message.jaro_distance!/3","doc":"Returns the [Jaro distance](https://en.wikipedia.org/wiki/Jaro–Winkler_distance)\nbetween two messages or raises.\n\nThis allows for fuzzy matching of message\nwhich can be helpful when a message string\nis changed but the semantics remain the same.","ref":"Cldr.Message.html#jaro_distance!/3"},{"type":"function","title":"Arguments - Cldr.Message.jaro_distance!/3","doc":"* `message1` is a CLDR message in binary form.\n\n* `message2` is a CLDR message in binary form.\n\n* `options` is a keyword list of options. The\n  default is `[]`.","ref":"Cldr.Message.html#jaro_distance!/3-arguments"},{"type":"function","title":"Options - Cldr.Message.jaro_distance!/3","doc":"* `:trim` determines if the message is trimmed\n  of whitespace before formatting. The default is\n  `false`.","ref":"Cldr.Message.html#jaro_distance!/3-options"},{"type":"function","title":"Returns - Cldr.Message.jaro_distance!/3","doc":"* `distance` where `distance` is a float value between 0.0\n  (equates to no similarity) and 1.0 (is an\n  exact match) representing Jaro distance between `message1`\n  and `message2` or\n\n* raises an exception","ref":"Cldr.Message.html#jaro_distance!/3-returns"},{"type":"function","title":"Examples - Cldr.Message.jaro_distance!/3","doc":"iex> Cldr.Message.jaro_distance! \"{greetings} to you!\", \"{greeting} to you!\"\n    0.9824561403508771","ref":"Cldr.Message.html#jaro_distance!/3-examples"},{"type":"macro","title":"Cldr.Message.t/1","doc":"Returns the translation of the given ICU-formatted\nmessage string.\n\nAny placeholders are replaced with the value of variables\nalready in scope at the time of compilation.\n\n`t/1` is a wrapper around the `gettext/2` macro\nwhich should therefore be imported from a `Gettext`\nbackend prior to calling `t/1`.","ref":"Cldr.Message.html#t/1"},{"type":"macro","title":"Arguments - Cldr.Message.t/1","doc":"* `message` is an ICU format message string.","ref":"Cldr.Message.html#t/1-arguments"},{"type":"macro","title":"Returns - Cldr.Message.t/1","doc":"* A translated string.","ref":"Cldr.Message.html#t/1-returns"},{"type":"macro","title":"Cldr.Message.t/2","doc":"Returns the translation of the given ICU-formatted\nmessage string.\n\n`t/2` is a wrapper around the `gettext/2` macro\nwhich should therefore be imported from a `Gettext`\nbackend prior to calling `t/2`.","ref":"Cldr.Message.html#t/2"},{"type":"macro","title":"Arguments - Cldr.Message.t/2","doc":"* `message` is an ICU format message string.\n\n* `bindings` is a keyword list or map of bindings used\n  to replace placeholders in the message.","ref":"Cldr.Message.html#t/2-arguments"},{"type":"macro","title":"Returns - Cldr.Message.t/2","doc":"* A translated string.","ref":"Cldr.Message.html#t/2-returns"},{"type":"type","title":"Cldr.Message.arguments/0","doc":"","ref":"Cldr.Message.html#t:arguments/0"},{"type":"type","title":"Cldr.Message.bindings/0","doc":"","ref":"Cldr.Message.html#t:bindings/0"},{"type":"type","title":"Cldr.Message.message/0","doc":"","ref":"Cldr.Message.html#t:message/0"},{"type":"type","title":"Cldr.Message.options/0","doc":"","ref":"Cldr.Message.html#t:options/0"},{"type":"module","title":"Cldr.Message.Backend","doc":"","ref":"Cldr.Message.Backend.html"},{"type":"function","title":"Cldr.Message.Backend.define_message_module/1","doc":"","ref":"Cldr.Message.Backend.html#define_message_module/1"},{"type":"function","title":"Cldr.Message.Backend.expand_to_binary!/2","doc":"Expands the given `message` in the given `env`, raising if it doesn't expand to\na binary.","ref":"Cldr.Message.Backend.html#expand_to_binary!/2"},{"type":"module","title":"Cldr.Message.Interpreter","doc":"","ref":"Cldr.Message.Interpreter.html"},{"type":"function","title":"Cldr.Message.Interpreter.format_list/3","doc":"Formats a parsed ICU message into an iolist.","ref":"Cldr.Message.Interpreter.html#format_list/3"},{"type":"function","title":"Cldr.Message.Interpreter.format_list!/3","doc":"","ref":"Cldr.Message.Interpreter.html#format_list!/3"},{"type":"module","title":"Cldr.Message.Parser","doc":"Implements a parser for the [ICU Message format](http://userguide.icu-project.org/formatparse/messages)","ref":"Cldr.Message.Parser.html"},{"type":"function","title":"Cldr.Message.Parser.message/2","doc":"Parses the given `binary` as message.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the message (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Cldr.Message.Parser.html#message/2"},{"type":"function","title":"Options - Cldr.Message.Parser.message/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Cldr.Message.Parser.html#message/2-options"},{"type":"function","title":"Cldr.Message.Parser.parse/2","doc":"","ref":"Cldr.Message.Parser.html#parse/2"},{"type":"function","title":"Cldr.Message.Parser.parse!/2","doc":"","ref":"Cldr.Message.Parser.html#parse!/2"},{"type":"function","title":"Cldr.Message.Parser.plural_message/2","doc":"Parses the given `binary` as plural_message.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the plural_message (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Cldr.Message.Parser.html#plural_message/2"},{"type":"function","title":"Options - Cldr.Message.Parser.plural_message/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Cldr.Message.Parser.html#plural_message/2-options"},{"type":"module","title":"Cldr.Message.Sigil","doc":"Implements sigil `~M` to canonicalize an\nICU message.\n\nICU messages allow for whitespace to be used to\nformat the message for developer and translator readability.\nAt the same time, `gettext` uses the message string\nas a key when resolving translations.\n\nTherefore a developer or translator that modifies\nthe message for readability may unintentionally\ncreate a new message rather than replace the old one\nsimply because the message strings don't match exactly.\n\nIt is possible to use the `fuzzy` option to the task\n`mix gettext.extract` however this may not be the desired\nbehaviour either.\n\nThe sigil `~M` therefore introduces a way for the developer\nto ensure the message is in a canonical format during\ncompilation and therefore both error check the message format\nand ensure the message is in a canonical form irrespective\nof developer formatting.","ref":"Cldr.Message.Sigil.html"},{"type":"macro","title":"Cldr.Message.Sigil.sigil_M/2","doc":"Handles the sigil `~M` for ICU message strings.\n\nIt returns a a canonically formatted string without\ninterpolations and without escape characters, except\nfor the escaping of the closing sigil character\nitself.\n\nA canonically formatted string is pretty-printed by\ndefault returning a potentially multi-line\nstring.  This is intended to produce a result which is\neasier to comprehend for translators.\n\nThe modifier `u` can be applied to return\na non-pretty-printed string.","ref":"Cldr.Message.Sigil.html#sigil_M/2"},{"type":"macro","title":"Modifi - Cldr.Message.Sigil.sigil_M/2","doc":"","ref":"Cldr.Message.Sigil.html#sigil_M/2-modifi"},{"type":"macro","title":"Examples - Cldr.Message.Sigil.sigil_M/2","doc":"iex> ~m(An ICU message)\n    \"An ICU message\"\n\nHowever, if you want to re-use the sigil character itself on\nthe string, you need to escape it:\n\n    iex> ~M((\\))\n    \"()\"","ref":"Cldr.Message.Sigil.html#sigil_M/2-examples"},{"type":"exception","title":"Cldr.Message.BindError","doc":"Exception raised when interpreting a\nmessage and a binding is missing","ref":"Cldr.Message.BindError.html"},{"type":"exception","title":"Cldr.Message.ParseError","doc":"Exception raised when parsing an ICU message format","ref":"Cldr.Message.ParseError.html"},{"type":"exception","title":"Cldr.Message.PositionalArgsNotPermitted","doc":"Exception raised when parsing an ICU message format\nand positional arguments are not permitted","ref":"Cldr.Message.PositionalArgsNotPermitted.html"},{"type":"extras","title":"Cldr Messages","doc":"# Cldr Messages\n\n![Build status](https://github.com/elixir-cldr/cldr_messages/actions/workflows/ci.yml/badge.svg)\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_cldr_messages.svg)](https://hex.pm/packages/ex_cldr_messages)\n[![Hex.pm](https://img.shields.io/hexpm/dw/ex_cldr_messages.svg?)](https://hex.pm/packages/ex_cldr_messages)\n[![Hex.pm](https://img.shields.io/hexpm/l/ex_cldr_messages.svg)](https://hex.pm/packages/ex_cldr_messages)","ref":"readme.html"},{"type":"extras","title":"Installation - Cldr Messages","doc":"```elixir\ndef deps do\n  [\n    {:ex_cldr_messages, \"~> 1.0\"}\n  ]\nend\n```\n\nDocumentation is at [https://hexdocs.pm/ex_cldr_messages](https://hexdocs.pm/ex_cldr_messages).","ref":"readme.html#installation"},{"type":"extras","title":"Introduction - Cldr Messages","doc":"Implements the [ICU Message Format](https://unicode-org.github.io/icu/userguide/format_parse/messages) for Elixir.\n\nIn any application that addresses audiences from different cultures, the need arises to support the presentation of user interfaces, messages, alerts and other content in the appropriate language for a user.\n\nFor [nearly 30 years](https://www.gnu.org/software/gettext/manual/gettext.html#gettext) the go-to solution for this requirement in many computer languages is [gettext](https://www.gnu.org/software/gettext). There is a full-featured [implementation for Elixir](https://hex.pm/packages/gettext) that is installed by default with [Phoenix](https://hex.pm/packages/phoenix) with over 10,000,000 downloads.\n\nGiven the maturity and widespread adoption of `Gettext`, why implement another format? Leveraging the content from the [Unicode CLDR](https://cldr.unicode.org) project we can address some of the shortcomings of `Gettext`. A good description of motivations and differences can be found in [this presentation](https://docs.google.com/presentation/d/1ZyN8-0VXmod5hbHveq-M1AeQ61Ga3BmVuahZjbmbBxo/pub?start=false&loop=false&delayms=3000&slide=id.g1bc43a82_2_14) by Mark Davis from Google in 2012.\n\nTwo specific shortcomings that the ICU message format addresses:\n\n#","ref":"readme.html#introduction"},{"type":"extras","title":"Grammatical Gender - Cldr Messages","doc":"Many languages inflect in gender specific way. One example in French might be:\n\n```elixir\n# You are the only participant for a male and female\nVous êtes le seul participant\nVous êtes la seule participante\n\n# Married for a male and a female\nMarié\nMariée\n```\n\nIn `Gettext` this requires individual messages and conditional code in the application in order to present the correct message to an audience.  This is compounded by the fact that some languages have more than two [grammatical genders](https://en.wikipedia.org/wiki/Grammatical_gender) (most have two and four but some are attested with up to 20).\n\nThe ICU message format provides a mechanism (the [select format](https://support.crowdin.com/icu-message-syntax/#select) that helps translator and UX designers implement a single message to easily encapsulate messages conditional on grammatical gender (or any other selector).\n\n#","ref":"readme.html#grammatical-gender"},{"type":"extras","title":"Standardised plural rules - Cldr Messages","doc":"Although `Gettext` supports pluralization for messages through the [Gettext.Plural module in Elixir](https://hexdocs.pm/gettext/Gettext.Plural.html) and the `Gettext` functions like `Gettext.ngettext/4`, the plural rules for a language have to be implemented for each message. Given the wide differences in how plural forms are structured in different languages this can be a material challenge.  For example:\n\n* English has two plural forms: singular and plural\n* French applies the singular rule to two values (0 and 1) and a plural form to larger groupings\n* Japanese does not differentiate\n* Russian has 4 categories\n* Arabic has 6 categories\n\nSince CLDR has a strong set of pluralization rules defined for ~500 locales, each of which is supported by [ex_cldr](https://hex.pm/packages/ex_cldr), the ICU message format can reuse these pluralization rules in a simple and consistent fashion using the [plural format](https://support.crowdin.com/icu-message-syntax/#plural).","ref":"readme.html#standardised-plural-rules"},{"type":"extras","title":"Getting Started - Cldr Messages","doc":"In common with other [ex_cldr](https://hex.pm/packages/ex_cldr)-based libraries, a `Cldr.Message` provider module needs to be configured as part of a [CLDR backend](https://hexdocs.pm/ex_cldr/readme.html#backend-module-configuration) module definitiom. For example:\n```elixir\n# Note the configuration of the Cldr.Message provider module\n# The provider Cldr.Number is required, all the others are optional\n# but if configured provide easy formatting of dates, times, lists and units\ndefmodule MyApp.Cldr do\n  use Cldr,\n    locales: [\"en\", \"fr\", \"ja\", \"he\", \"th\", \"ar\"],\n    default_locale: \"en\",\n    providers: [Cldr.Number, Cldr.DateTime, Cldr.Unit, Cldr.List, Cldr.Calendar, Cldr.Message]\nend\n```","ref":"readme.html#getting-started"},{"type":"extras","title":"Message format overview - Cldr Messages","doc":"ICU message formats are Elixir strings with embedded formatting directives inserted between `{}`. Some examples:\n\n```elixir\n# Insert the binding `name` into the string\n\"My name is {name}\"\n\n# Insert a date, formatting in a localized `short` format plus a localized plural form\n# for the binding `num_photos`\n\"On {taken_date, date, short} {name} took {num_photos, plural,\n  =0 {no photos.}\n  =1 {one photo.}\n  other {# photos.}}\"\n\n# Insert localized messages based upon the gender of the audience with\n# appropriate localized plural forms\n\"{gender_of_host, select,\n  female {\n    {num_guests, plural, offset: 1\n      =0 {{host} does not give a party.}\n      =1 {{host} invites {guest} to her party.}\n      =2 {{host} invites {guest} and one other person to her party.}\n      other {{host} invites {guest} and # other people to her party.}}}\n  male {\n    {num_guests, plural, offset: 1\n      =0 {{host} does not give a party.}\n      =1 {{host} invites {guest} to his party.}\n      =2 {{host} invites {guest} and one other person to his party.}\n      other {{host} invites {guest} and # other people to his party.}}}\n  other {\n    {num_guests, plural, offset: 1\n      =0 {{host} does not give a party.}\n      =1 {{host} invites {guest} to their party.}\n      =2 {{host} invites {guest} and one other person to their party.}\n      other {{host} invites {guest} and # other people to their party.}}}\n}\"\n```\nFurther information on ICU message formats is [here](message_format.html).","ref":"readme.html#message-format-overview"},{"type":"extras","title":"Message formatting API - Cldr Messages","doc":"Using the above messages as examples:\n\n```elixir\niex> Cldr.Message.format! \"My name is {name}\", name: \"Kip\"\n\"My name is Kip\"\n\niex> Cldr.Message.format!  \"On {taken_date, date, short} {name} took {num_photos, plural,\n       =0 {no photos.}\n       =1 {one photo.}\n       other {# photos.}}\", taken_date: Date.utc_today, name: \"Kip\", num_photos: 10\n\"On 8/26/19 Kip took 10 photos.\"\n```\n\nAs of `ex_cldr_messages` version 0.3.0 a macro form is introduced which parses the message at compile time in order to optimize performance at run time. To use the macro, a backend module must be imported (or required) into a module that uses formatting.  For example:\n\n```elixir\ndefmodule SomeModule do\n  # Import a  .Cldr.Message module\n  import MyApp.Cldr.Message\n\n  def my_function do\n    format(\"this is a string with a param {param}\", param: 3)\n  end\nend\n```","ref":"readme.html#message-formatting-api"},{"type":"extras","title":"Gettext integration - Cldr Messages","doc":"As of [Gettext 0.19](https://hex.pm/packages/gettext/0.19.0), `Gettext` supports user-defined [interpolation modules](https://hexdocs.pm/gettext/Gettext.html#module-backend-configuration). This makes it easy to combine the power of ICU message formats with the broad `gettext` ecosystem and the inbuilt support for `gettext` in [Phoenix](https://hex.pm/packages/phoenix).  The documentation for [Gettext](https://hexdocs.pm/gettext/Gettext.html#content) should be followed with considerations in mind:\n\n1. A Gettext backend module should use the `:interpolation` option defined referring to the `ex_cldr_messages` backend you have defined.\n2. The message format is in the ICU message format (instead of the Gettext format).\n\n#","ref":"readme.html#gettext-integration"},{"type":"extras","title":"Defining a Gettext Interpolation Module - Cldr Messages","doc":"Any [ex_cldr](https://hex.pm/packages/ex_cldr) [backend module](https://hexdocs.pm/ex_cldr/readme.html#backend-module-configuration) that has a `Cldr.Message` provider configured can be used as an interpolation module. Here is an example:\n```elixir\n# CLDR backend module\ndefmodule MyApp.Cldr do\n  use Cldr,\n    locales: [\"en\", \"fr\", \"ja\", \"he\", \"th\", \"ar\"],\n    default_locale: \"en\",\n    providers: [Cldr.Number, Cldr.DateTime, Cldr.Unit, Cldr.List, Cldr.Calendar, Cldr.Message],\n    gettext: MyApp.Gettext,\n    message_formats: %{\n      USD: [format: :long]\n    }\nend\n\n# Define an interpolation module for ICU messages\ndefmodule MyApp.Gettext.Interpolation do\n  use Cldr.Gettext.Interpolation, cldr_backend: MyApp.Cldr\n\nend\n\n# Define a gettext module with ICU message interpolation\ndefmodule MyApp.Gettext do\n  use Gettext, otp_app: :ex_cldr_messages, interpolation: MyApp.Gettext.Interpolation\nend\n\n```\nNow you can proceed to use `Gettext` in the normal manner, most typically with the `gettext/3` macro.","ref":"readme.html#defining-a-gettext-interpolation-module"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2019 Kip Cole\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Cldr_Messages v1.0.0 - Changelog","doc":"This is the changelog for Cldr_Messages v1.0.0 released on October 5th, 2023.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\nSince the API is stable, this becomes the 1.0 release.\n\n#","ref":"changelog.html#cldr_messages-v1-0-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `Cldr.Message.Print.to_string/2` to force a deterministic order of `choice`s.\n\n* Fix extracting variable bindings from `simple_format`. Thanks to @AlexisChavez97 for the report and the PR.  Closes #16.\n\n* Bump Elixir minimum required version to 1.11 to match the rest of `ex_cldr`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr_Messages v0.14.1 - Changelog","doc":"This is the changelog for Cldr_Messages v0.14.1 released on October 4th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-14-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Returns a partially formatted message if bindings are missing. The message will be interpolated as far as possibly making the error message more meaningful for end user consumption. Thanks to @maennchen for the PR.  Closes #14.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr_Messages v0.14.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.14.0 released on February 22nd, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-14-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Updates to [ex_cldr version 2.26.0](https://hex.pm/packages/ex_cldr/2.26.0) which uses atoms for locale names and rbnf locale names. This is consistent with other elements of `t:Cldr.LanguageTag` where atoms are used where the cardinality of the data is fixed and relatively small and strings where the data is free format.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.13.2 - Changelog","doc":"This is the changelog for Cldr_Messages v0.13.2 released on January 14th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-13-2"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixes formatting when there is no `:message_formats` configured on a backend. Closes #11, again.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr_Messages v0.13.1 - Changelog","doc":"This is the changelog for Cldr_Messages v0.13.1 released on January 13th, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-13-1"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `Cldr.Message.Interpolate.compile_interpolate/3` to ensure that the parsed message is escaped AST. Closes #11.\n\n* Fix passing RBNF format names as number format styles.\n\n* Fix converting strings to atoms at compile time. Previously these uses `String.to_existing_atom/1` however the compilation graph sometimes means that the expected atoms are not available. Therefore `String.to_atom/1` is now called. As a result, it is very important to note that allowing arbitrary message formats into the system could open an attack vector to exhaust the atom table.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr_Messages v0.13.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.13.0 released on January 2nd, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-13-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Don't add `:gettext` to `Mix.compilers/0` because it gets inherited into client applications and we want `Gettext` to remain optional. Thanks to @trarbr for the report.\n\n* Fix typos. Thanks as always to @kianmeng","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add an Elixir formatter plugin for `sigil_M`. For example in your `.formatter.exs` file:\n```elixir\n[\n  inputs: [\"mix.exs\", \"{config,lib,test,mix}/**/*.{ex,exs}\"],\n  locals_without_parens: [docp: 1, defparsec: 2, defparsec: 3],\n  plugins: [Cldr.Formatter.Plugin]\n]\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.12.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.12.0 released on January 1st, 2022.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-12-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"The primary focus of this release is gettext integration. The effort is kindly driven by @maennchen and the majority of the work is his.\n\n* Add `Cldr.Message.format_to_iolist/3` which formats a binary message into an iolist. This helps with incrementally binding placeholders where some may be bound at compile time and others at runtime. It also makes integration with `gettext` simpler since this function returns a list of completed bindings and a list of bindings that were not found.\n\n* Adds an implementation of `Gettext.Interpolation` to support `gettext` integration.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.11.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.11.0 released on May 23rd, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-11-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Cldr.Message.bindings/1` to extract the names of variable bindings for a message\n\n* Add option `:allow_positional_args` to `Cldr.Message.format/3`. The default is `true`.\n\n* Add argument `allow_positional_args?` to `Cldr.Message.Parser.parse/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.10.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.10.0 released on April 8th, 2021.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-10-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Use `Cldr.default_backend!/0` if available","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.9.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.9.0 released on November 18th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-9-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Supports number arguments to be either a number or a tuple of the form `{number, keyword_list_of_formatting_options}`. This provides an escape hatch for developers to influence formatting of the number beyond the textual format of the message.  For example:\n\n```elixir\n# Uses the currency for the current locale\niex> Cldr.Message.format \"this is {one, number, currency}\", one: 1\n{:ok, \"this is $1.00\"}\n\n# Forces the :MXP currency\niex> Cldr.Message.format \"this is {one, number, currency}\", one: {1, currency: :MXP}\n{:ok, \"this is MXP 1.00\"}\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.8.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.8.0 released on September 27th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-8-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Fix `Cldr.Message.Print.to_string/2` to omit blank spaces at the end of lines when pretty printing.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.7.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.7.0 released on September 26th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-7-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Allow `nimble_parsec` versions `~> 0.5 or ~> 1.0`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.6.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.6.0 released on September 26th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-6-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Cldr.Message.canonical_message/2` that converts a string message into a canonical form. This allows for fuzzy matching between two messages that may have different formatting (this is possible since the CLDR message format allows for non-formatting whitespace in parts of the syntax).\n\n* Add documentation for some of the key functions. Much more documentation required before 1.0 release.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.5.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.5.0 released on September 22nd, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-5-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Adds compile time checking that bindings are provided to the `format/3` macro wherever possible\n\n* Supports later versions of `ex_cldr` and friends, `ex_money` as well as Elixir 1.11 without warnings","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.4.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.4.0 released on August 29th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-4-0"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Conditionally compile functions that depend on optional dependencies","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr_Messages v0.3.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.3.0 released on August 29th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-3-0"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Standardize on the `Cldr.Message.format/3` as the public api. `Cldr.Message.to_string/3` is removed.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add the macro ` .Cldr.Message.format/3` to parse messages at compile time as a way to optiise performance at runtime. To use it add `import  .Cldr.Message` to your module and use `format/3`.  An example:\n\n```elixir\ndefmodule SomeModule do\n  import MyApp.Cldr.Message\n\n  def my_function do\n    format(\"this is a string with a param {param}\", param: 3)\n  end\nend\n```\n\n* Add `Cldr.Message.format_to_list/3` formats to an `io_list`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix dialyzer warnings.  There are some warnings from combinators that will require `nimble_parsec` version 0.5.2 to be published before they are resolved.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Cldr_Messages v0.2.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.2.0 released on August 27th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n#","ref":"changelog.html#cldr_messages-v0-2-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Uses `Cldr.Number.to_string/3` to format simple arguments that are numeric (integer, float and decimal).  This gives a localised number format. An example:\n\n```elixir\niex> Cldr.Message.to_string \"You have {number} jelly beans\", number: 1234\n\"You have 1,234 jelly beans\"\n```\n\n* Similarly applies localized formatting for dates, times, datetimes.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Cldr_Messages v0.1.0 - Changelog","doc":"This is the changelog for Cldr_Messages v0.1.0 released on August 26th, 2019.  For older changelogs please consult the release tag on [GitHub](https://github.com/elixir-cldr/cldr_messages/tags)\n\n* Initial release.  This release implements `Cldr.Message.to_string/3` and `Cldr.Message.format/3`\n\nThis initial release is the basis for building a complete message localization solution as an alternative to [Gettext](https://hex.pm/packages/gettext).  There is a long way to go until that is accomplished.","ref":"changelog.html#cldr_messages-v0-1-0"},{"type":"extras","title":"ICU Message Specification","doc":"# ICU Message Specification\n\nMessages are user-visible strings, often with variable elements like names,\nnumbers and dates. Message strings are typically translated into the different\nlanguages of a UI, and translators move around the variable elements according\nto the grammar of the target language.\n\nFor this to work in many languages, a message has to be written and translated\nas a single unit, typically a string with placeholder syntax for the variable\nelements. If the user-visible string were concatenated directly from fragments\nand formatted elements, then translators would not be able to rearrange the\npieces, and they would have a hard time translating each of the string\nfragments.\n\nThis document is an edited version of the [official ICU documentation](https://unicode-org.github.io/icu/userguide/format_parse/messages/) edited\nto reflect the implementation in [ex_cldr_messages](https://hex.pm/packages/ex_cldr_messages).","ref":"message_format.html"},{"type":"extras","title":"Format Overview - ICU Message Specification","doc":"The ICU Message Format uses message `\"pattern\"` strings with\nvariable-element placeholders enclosed in {curly braces}. The\nargument syntax can include formatting details, otherwise a\ndefault format is used.\n\n#","ref":"message_format.html#format-overview"},{"type":"extras","title":"Complex Argument Types - ICU Message Specification","doc":"Certain types of arguments select among several choices which are nested\n`Message Format` pattern strings. Keeping these choices together in one message\npattern string facilitates translation in context, by one single translator.\n(Commercial translation systems often distribute different messages to different\ntranslators.)\n\n*   Use a `\"plural\"` argument to select sub-messages based on a numeric value,\n    together with the plural rules for the specified language.\n*   Use a `\"select\"` argument to select sub-messages via a fixed set of keywords.\n*   Use of the old `\"choice\"` argument type is discouraged. It cannot handle\n    plural rules for many languages, and is clumsy for simple selection.\n\nIt is tempting to cover only a minimal part of a message string with a complex\nargument (e.g., plural). However, this is difficult for translators for two\nreasons: 1. They might have trouble understanding how the sentence fragments in\nthe argument sub-messages interact with the rest of the sentence, and 2. They\nwill not know whether and how they can shrink or grow the extent of the part of\nthe sentence that is inside the argument to make the whole message work for\ntheir language.\n\n**Recommendation:** If possible, use complex arguments as the outermost\nstructure of a message, and write **full sentences** in their sub-messages. If\nyou have nested select and plural arguments, place the **select** arguments\n(with their fixed sets of choices) on the **outside** and nest the plural\narguments (hopefully at most one) inside.\n\nFor example:\n\n```text\n\"{gender_of_host, select, \"\n  \"female {\"\n    \"{num_guests, plural, offset:1 \"\n      \"=0 {{host} does not give a party.}\"\n      \"=1 {{host} invites {guest} to her party.}\"\n      \"=2 {{host} invites {guest} and one other person to her party.}\"\n      \"other {{host} invites {guest} and # other people to her party.}}}\"\n  \"male {\"\n    \"{num_guests, plural, offset:1 \"\n      \"=0 {{host} does not give a party.}\"\n      \"=1 {{host} invites {guest} to his party.}\"\n      \"=2 {{host} invites {guest} and one other person to his party.}\"\n      \"other {{host} invites {guest} and # other people to his party.}}}\"\n  \"other {\"\n    \"{num_guests, plural, offset:1 \"\n      \"=0 {{host} does not give a party.}\"\n      \"=1 {{host} invites {guest} to their party.}\"\n      \"=2 {{host} invites {guest} and one other person to their party.}\"\n      \"other {{host} invites {guest} and # other people to their party.}}}}\"\n```\n\n**Note:** In a plural argument like in the example above, if the English message\nhas both `=0` and `=1` (up to `=offset`+1) then it does not need a \"`one`\"\nvariant because that would never be selected. It does always need an \"`other`\"\nvariant.\n\n**Note:** *The translation system and the translator together need to add\n[\"`one`\", \"`few`\" etc. if and as necessary per target\nlanguage](http://cldr.unicode.org/index/cldr-spec/plural-rules).*\n\n#","ref":"message_format.html#complex-argument-types"},{"type":"extras","title":"Quoting/Escaping - ICU Message Specification","doc":"If syntax characters occur in the text portions, then they need to be quoted by\nenclosing the syntax in pairs of ASCII apostrophes. A pair of ASCII apostrophes\nalways represents one ASCII apostrophe, similar to `%%` in `printf` representing one `%`,\nalthough this rule still applies inside quoted text. (\"`This '{isn''t}' obvious`\" → \"`This {isn't} obvious`\")\n\n*   Recommendation: Use the real apostrophe (single quote) character `’` (U+2019)\n    for human-readable text, and use the ASCII apostrophe `'` (U+0027) only in\n    program syntax, like quoting in Message Format. See the annotations for\n    U+0027 Apostrophe in The Unicode Standard.\n\n#","ref":"message_format.html#quoting-escaping"},{"type":"extras","title":"Argument formatting - ICU Message Specification","doc":"Arguments are formatted according to their type, using the default `ex_cldr`\nformatters for those types, unless otherwise specified. For unknown types the\nthe function `to_string/0` will be called. Formatters are supported for:\n\n* Numbers (integer, float and Decimal) through [ex_cldr_numbers](https://hex.pm/packages/ex_cldr_numbers)\n* Dates, Times and DateTimes through [ex_cldr_dates_times](https://hex.pm/packages/ex_cldr_dates_times)\n* Units of measure through [ex_cldr_units](https://hex.pm/packages/ex_cldr_units)\n* Money through [ex_money](https://hex.pm/packages/ex_money)\n\nThere are also several ways to control the formatting.\n\n##","ref":"message_format.html#argument-formatting"},{"type":"extras","title":"Predefined styles (recommended) - ICU Message Specification","doc":"You can specify the `arg_style` to be one of the predefined values `short`, `medium`,\n`long`, `full` (to get one of the standard forms for dates / times) and `integer`,\n`currency`, `percent` (for number formatting).\n\n##","ref":"message_format.html#predefined-styles-recommended"},{"type":"extras","title":"Format the parameters separately (recommended) - ICU Message Specification","doc":"You can format the parameter as you need **before** calling `Cldr.Message.format/3`, and\nthen passing the resulting string as a parameter to `Cldr.Message.format/3`.\n\nThis offers maximum control, and is preferred to using custom format objects\n(see below).\n\n##","ref":"message_format.html#format-the-parameters-separately-recommended"},{"type":"extras","title":"String patterns (discouraged) - ICU Message Specification","doc":"These can be used for numbers, dates, and times, but they are locale-sensitive,\nand they therefore would need to be localized by your translators, which adds\ncomplexity to the localization, and placeholder details are often not accessible\nby translators. If such a pattern is not localized, then users see confusing\nformatting. Consider using skeletons instead of patterns in your message\nstrings.\n\nAllowing translators to localize date patterns is error-prone, as translators\nmight make mistakes (resulting in invalid CLDR date formatter syntax).\nAlso, CLDR provides curated patterns for many locales, and using your own pattern means\nthat you don't benefit from that CLDR data and the results will likely be\ninconsistent with the rest of the patterns that CLDR uses.\n\nIt is also a bad internationalization practice, because most companies only\ntranslate into \"generic\" versions of the languages (French, or Spanish, or\nArabic). So the translated patterns get used in tens of countries. On the other\nhand, skeletons are localized according to the locale, which\nshould include regional variants (e.g., “fr-CA”).","ref":"message_format.html#string-patterns-discouraged"},{"type":"extras","title":"Message Format Syntax - ICU Message Specification","doc":"`ex_cldr_messages` prepares strings for display to users, with optional arguments (variables/placeholders). The arguments can occur in any order, which is necessary for translation into languages with different grammars.\n\nA message is constructed from a pattern string with arguments in {curly braces} which will be replaced by formatted values.\n\n* Arguments can be named (using identifiers) or numbered (using small ASCII-digit integers).\n\n* An argument might not specify any format type. In this case, a Number value is formatted with a default (for the locale) Number Format, a Date value is formatted with a default (for the locale) Date Format, and so on for a Unit, Money and Curency. For any other value its `to_string/0` is called.\n\n* An argument might specify a \"simple\" type for which the specified Format object is created, cached and used.\n\n* An argument might have a \"complex\" type with nested MessageFormat sub-patterns. During formatting, one of these sub-messages is selected according to the argument value and recursively formatted.\n\nWhen formatting, `Cldr.Message.format/3` takes a collection of argument values and writes an output string. The argument values may be passed as a list (when the pattern contains only numbered arguments) or as a Map (which works for both named and numbered arguments).\n\nEach argument is matched with one of the input values by list index or map key and formatted according to its pattern specification. A numbered pattern argument is matched with a map key that contains that number as an ASCII-decimal-digit string (without leading zero).\n\n#","ref":"message_format.html#message-format-syntax"},{"type":"extras","title":"Patterns and Their Interpretation - ICU Message Specification","doc":"Message Format uses patterns of the following form:\n```\n message = messageText (argument messageText)*\n argument = noneArg | simpleArg | complexArg\n complexArg = choiceArg | pluralArg | selectArg | selectordinalArg\n\n noneArg = '{' argNameOrNumber '}'\n simpleArg = '{' argNameOrNumber ',' argType [',' argStyle] '}'\n choiceArg = '{' argNameOrNumber ',' \"choice\" ',' choiceStyle '}'\n pluralArg = '{' argNameOrNumber ',' \"plural\" ',' pluralStyle '}'\n selectArg = '{' argNameOrNumber ',' \"select\" ',' selectStyle '}'\n selectordinalArg = '{' argNameOrNumber ',' \"selectordinal\" ',' pluralStyle '}'\n\n choiceStyle: see ChoiceFormat\n pluralStyle: see PluralFormat\n selectStyle: see SelectFormat\n\n argNameOrNumber = argName | argNumber\n argName = [^[[:Pattern_Syntax:][:Pattern_White_Space:]]]+\n argNumber = '0' | ('1'..'9' ('0'..'9')*)\n\n argType = \"number\" | \"date\" | \"time\" | \"spellout\" | \"ordinal\" | \"duration\"\n argStyle = \"short\" | \"medium\" | \"long\" | \"full\" | \"integer\" | \"currency\" | \"percent\" | argStyleText\n```\n\nMessages can contain quoted literal strings including syntax characters. A quoted literal string begins with an ASCII apostrophe and a syntax character (usually a {curly brace}) and continues until the next single apostrophe. A double ASCII apostrophe inside or outside of a quoted string represents one literal apostrophe.\nQuotable syntax characters are the `{curly braces}` in all message parts, plus the `#` sign in a message immediately inside a `pluralStyle`, and the '|' symbol in a messageText immediately inside a `choiceStyle`.\n\nIn argStyleText, every single ASCII apostrophe begins and ends quoted literal text, and unquoted {curly braces} must occur in matched pairs.\n\nRecommendation: Use the real apostrophe (single quote) character \\\\u2019 for human-readable text, and use the ASCII apostrophe (\\\\u0027 ' ) only in program syntax, like quoting in MessageFormat. See the annotations for U+0027 Apostrophe in The Unicode Standard.\n\nThe choice argument type is deprecated. Use plural arguments for proper plural selection, and select arguments for simple selection among a fixed set of choices.\n\n#","ref":"message_format.html#patterns-and-their-interpretation"},{"type":"extras","title":"Examples - ICU Message Specification","doc":"See the examples in the [README](/README.md)","ref":"message_format.html#examples"}],"content_type":"text/markdown"}